﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Threading;

using Dundas.Charting.WinControl;

using CoboltMonitor.Laser;
using CoboltMonitor.Gui.Logic;

namespace CoboltMonitor.Gui
{
	public partial class IPLog2 : Form
	{
        public delegate void UpdateSerialData(Object objSender, LaserUpdatedEvent dataReceivedEventArgs);
        private IPLog2Logic ipLogLogic;
        public IPLog2(LaserCommon laser, bool mA)
		{
			InitializeComponent();

            ipLogLogic = new IPLog2Logic(laser);

            if (mA)
                setUpmA();
            
            ipLogLogic.SamplesInterval = (int)numericUpDownSamples.Value;
            ipLogLogic.LogInterval = (float)numericUpDownDelay.Value;
            ipLogLogic.StartCurrent = (float)numericUpDownStartCurrent.Value;
            ipLogLogic.EndCurrent = (float)numericUpDownEndCurrent.Value;
            laser.LaserUpdatedEvent += new EventHandler<LaserUpdatedEvent>(ResponseReceived);
        }

        private void setUpmA()
        {
            label7.Text = "Start current (mA)";
            label8.Text = "End current (mA)";
            numericUpDownStartCurrent.Increment *= 1000;
            numericUpDownStartCurrent.Maximum *= 1000;
            numericUpDownStartCurrent.Minimum *= 1000;
            numericUpDownStartCurrent.Value *= 1000;
            numericUpDownStartCurrent.DecimalPlaces = 0;

            numericUpDownEndCurrent.Increment *= 1000;
            numericUpDownEndCurrent.Maximum *= 1000;
            numericUpDownEndCurrent.Minimum *= 1000;
            numericUpDownEndCurrent.Value *= 1000;
            numericUpDownEndCurrent.DecimalPlaces = 0;

            chart1.ChartAreas["Default"].AxisX.LabelStyle.Format = "N1";
            chart1.ChartAreas["Default"].AxisX.Title = "Current(mA)";
        }
		/// <summary>
		/// Collects the data.
		/// </summary>
		/// <param name="curr">The current value of the sample.</param>
		/// <param name="pwr">The power value of the sample.</param>
		public void CollectData()
		{
            if (!ipLogLogic.CollectData())
                return;

            // Only collect specified number of samples.
            if (ipLogLogic.Samples >= ipLogLogic.SamplesInterval)
            {
                ipLogLogic.Stop();
                Stop();
            }
            else
            {
                ipLogLogic.SetNextCurrent();
            } 
            
            labelSample.Text = ipLogLogic.Samples.ToString();
			chart1.Series["Series1"].Points.AddXY(ipLogLogic.GetCurrent(ipLogLogic.Samples - 1), ipLogLogic.GetPower(ipLogLogic.Samples - 1));
			chart1.Invalidate();
		}

		/// <summary>
		/// Handles the Load event of the IPLog control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void IPLog2_Load(object sender, EventArgs e)
		{
			Stop();
            ipLogLogic.Stop();

			chart1.ChartAreas["Default"].AxisX.Minimum = (double)numericUpDownStartCurrent.Value;
			chart1.ChartAreas["Default"].AxisX.Maximum = (double)numericUpDownEndCurrent.Value;
        }

		/// <summary>
		/// Handles the FormClosing event of the IPLog control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
		private void IPLog2_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (e.CloseReason == CloseReason.UserClosing)
				e.Cancel = true;

			Hide();
		}

		/// <summary>
		/// Starts logging I-P data.
		/// </summary>
		private void Start()
		{
            ipLogLogic.Start();
            chart1.Series[0].Points.Clear();

			numericUpDownStartCurrent.Enabled = false;
			numericUpDownEndCurrent.Enabled = false;
			numericUpDownDelay.Enabled = false;
			numericUpDownSamples.Enabled = false;

			buttonStart.Enabled = false;
			buttonAbort.Enabled = true;
			buttonExport.Enabled = false;

		}

		/// <summary>
		/// Stops logging I-P data.
		/// </summary>
		private void Stop()
		{
			numericUpDownStartCurrent.Enabled = true;
			numericUpDownEndCurrent.Enabled = true;
			numericUpDownDelay.Enabled = true;
			numericUpDownSamples.Enabled = true;

			buttonAbort.Enabled = false;
			buttonStart.Enabled = true;

            if (ipLogLogic.HasData())
				buttonExport.Enabled = true;
			else
				buttonExport.Enabled = false;
		}

		/// <summary>
		/// Handles the Click event of the buttonStart control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void buttonStart_Click(object sender, EventArgs e)
		{
            ipLogLogic.Start();
			Start();
		}

		/// <summary>
		/// Handles the Click event of the buttonAbort control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void buttonAbort_Click(object sender, EventArgs e)
		{
            ipLogLogic.Abort();
            ipLogLogic.Stop();
            Stop();
		}

		/// <summary>
		/// Handles the Click event of the buttonExport control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void buttonExport_Click(object sender, EventArgs e)
		{
            if (!ipLogLogic.HasData())
				return;

            Stream myStream = null;

			saveFileDialog1.Filter = "CSV (Comma delimited)|*.csv|All files (*.*)|*.*";
			saveFileDialog1.FilterIndex = 0;

			if (saveFileDialog1.ShowDialog() == DialogResult.OK)
			{
				try
				{
					if ((myStream = saveFileDialog1.OpenFile()) != null)
					{
						using (myStream)
						{
							StreamWriter writer = new StreamWriter(myStream);
							ipLogLogic.WriteCSV(writer);
							writer.Close();
						}
					}
				}
				catch (Exception ex)
				{
                    MessageBox.Show("Failed to save file", ex.Message);
				}
			}
		}

		/// <summary>
		/// Handles the Click event of the buttonClose control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		private void buttonClose_Click(object sender, EventArgs e)
		{
			Hide();
		}

        private void numericUpDownSamples_ValueChanged(object sender, EventArgs e)
        {
            ipLogLogic.SamplesInterval = (int)numericUpDownSamples.Value;
        }

        private void numericUpDownDelay_ValueChanged(object sender, EventArgs e)
        {
            ipLogLogic.LogInterval = (float)numericUpDownDelay.Value;
        }

		private void numericUpDownStartCurrent_ValueChanged(object sender, EventArgs e)
		{
            ipLogLogic.StartCurrent = (float)numericUpDownStartCurrent.Value;
			chart1.ChartAreas["Default"].AxisX.Minimum = (double)numericUpDownStartCurrent.Value;
		}

		private void numericUpDownEndCurrent_ValueChanged(object sender, EventArgs e)
		{
            ipLogLogic.EndCurrent = (float)numericUpDownEndCurrent.Value;
			chart1.ChartAreas["Default"].AxisX.Maximum = (double)numericUpDownEndCurrent.Value;
		}

        private void ResponseReceived(object sender, LaserUpdatedEvent e)
        {
            if (this.InvokeRequired == true)
            {
                UpdateSerialData updateHandler = new UpdateSerialData(ResponseReceived);
                this.BeginInvoke(updateHandler, new Object[] { sender, e });
                return;
            }

            switch (e.CommandResponse.CommandId)
            {
                case CommandId.ReadAll:
                    CollectData();
                    break;
            }
        }

	}
}
