using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using CoboltMonitor.Laser;

namespace CoboltMonitor.Gui
{
	public partial class LogCliHelp : Form
	{
        #region Instance variables
        bool loggedIn;
        #endregion

        private ICollection<CommandInfo> _commands;

        public bool LoggedIn
        {
            set {loggedIn = value; }
        }
        public LogCliHelp(bool loggedin,ICollection<CommandInfo> commands)
		{
			InitializeComponent();
            loggedIn = loggedin;
            _commands = commands;
        }

		#region Control Events
		private void buttonClose_Click(object sender, EventArgs e)
		{
			Hide();
		}
		#endregion

        private void LogCliHelp_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
                e.Cancel = true;

            Hide();
        }

        private void FillHelpList()
        {
            richTextBoxCliHelp.Clear();
            int namlen = 0;

            // Find longest name command length.
            foreach (CommandInfo ci in _commands)
            {
                int len = ci.Name.Length;

                if (len > namlen)
                    namlen = len;
            }
            namlen += 2;

            foreach (CommandInfo ci in _commands)
            {
                if (ci.SecretHelp == false || loggedIn == true)
                {
                    Font font = richTextBoxCliHelp.SelectionFont;
                    Color color = richTextBoxCliHelp.SelectionColor;

                    richTextBoxCliHelp.SelectionFont = new Font("Courier New", 10, FontStyle.Bold);
                    richTextBoxCliHelp.SelectionColor = Color.Blue;
                    string name = ci.Name;
                    int len = name.Length;
                    name += "                                ".Substring(0, namlen - len);
                    richTextBoxCliHelp.AppendText(name);
                    richTextBoxCliHelp.SelectionFont = font;
                    richTextBoxCliHelp.SelectionColor = color;
                    richTextBoxCliHelp.AppendText(ci.Info + "\r");
                }
            }

            // Scroll to first command description.
            richTextBoxCliHelp.Find("?");
            richTextBoxCliHelp.DeselectAll();
        }

        private void LogCliHelp_Activated(object sender, EventArgs e)
        {
            FillHelpList();
        }

        public void AuthChg()
        {
            FillHelpList();
        }
	}
}