using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using CoboltMonitor.Laser;

namespace CoboltMonitor.Gui
{
	public partial class LogTerm : Form
	{
        public delegate void UpdateSerialData(Object objSender, LaserUpdatedEvent dataReceivedEventArgs);
        #region Instance variables
        private LogCliHelp _cliHelp;
        private LaserCommon laser;
        private bool loggedIn;
        private bool _logAll = false;
        #endregion


		#region Constructors
        public LogTerm(LaserCommon laser)
		{
			InitializeComponent();
            _cliHelp = new LogCliHelp(loggedIn,laser.GetCommandList());
            this.laser = laser;
            laser.LaserUpdatedEvent += new EventHandler<LaserUpdatedEvent>(UpdateGui);
		}
		#endregion

        #region Properties
        public bool LogAll
        {
            get { return _logAll; }
        }
        #endregion
  
		#region Public member functions

        public void LoggedIn_Received(object sender, LoggedIn e)
        {
            loggedIn = e.Logged;
            _cliHelp.LoggedIn = loggedIn;
        }

		public bool Connected
		{
			set
			{
				buttonSend.Enabled = value;
				textBoxAtCommand.Enabled = value;
				if (!value)
				{
					labelSendQueueLength.Text = "0";
				}
			}
		}

		public void SetQueueLength(int len)
		{
			labelSendQueueLength.Text = len.ToString();
		}

		public void LogString(string logMsg)
		{
			int ih = listBoxCommsLog.ItemHeight;
			int ni = listBoxCommsLog.Items.Count;
			int vh = listBoxCommsLog.Size.Height;
			int mi = vh / ih;

			if (listBoxCommsLog.Items.Count > 200) //(ni >= mi)
			{
			    listBoxCommsLog.Items.RemoveAt(0);
			}

			listBoxCommsLog.Items.Add(logMsg);
			listBoxCommsLog.SelectedIndex = listBoxCommsLog.Items.Count - 1;
		}

		public void SendCliCommand()
		{
            if (loggedIn || !laser.NeedsAuth(textBoxAtCommand.Text))
            {
                laser.SendRawCommand(this, textBoxAtCommand.Text);
            }
		}

        public void AuthChg()
        {
            if (_cliHelp.Visible)
                _cliHelp.AuthChg();
        }
		#endregion

		#region Form Events
		private void LogTerm_Activated(object sender, EventArgs e)
		{
            ICollection<CommandInfo> _commands = laser.GetCommandList();
            textBoxAtCommand.AutoCompleteCustomSource.Clear();
            foreach (CommandInfo ci in _commands)
            {
                if (ci.SecretHelp == false || loggedIn == true)
                    textBoxAtCommand.AutoCompleteCustomSource.Add(ci.Name);
            }

            if (laser.IsConnected())
			{
				buttonSend.Enabled = true;
				textBoxAtCommand.Enabled = true;
			}
			else
			{
				buttonSend.Enabled = false;
				textBoxAtCommand.Enabled = false;
			}
		}
		#endregion

		#region Control Events
		private void buttonSend_Click(object sender, EventArgs e)
		{
			SendCliCommand();
		}

		private void buttonClear_Click(object sender, EventArgs e)
		{
			listBoxCommsLog.Items.Clear();
		}

		private void buttonClose_Click(object sender, EventArgs e)
		{
			Hide();
		}

		private void cbLogAll_CheckedChanged(object sender, EventArgs e)
		{
			_logAll = cbLogAll.Checked;
		}

		private void textBoxAtCommand_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar == (char)Keys.Return)
			{
				SendCliCommand();
				e.Handled = true;
			}
		}

		private void buttonCliHelp_Click(object sender, EventArgs e)
		{
			if (!_cliHelp.Visible)
				_cliHelp.Show(this);
		}
		#endregion

		private void textBoxAtCommand_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyData == Keys.Return)
			{
				SendCliCommand();
				e.Handled = true;
			}
		}

		private void LogTerm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (e.CloseReason == CloseReason.UserClosing)
				e.Cancel = true;

			Hide();
		}

        private void UpdateGui(object sender, LaserUpdatedEvent e)
        {
            CommandResponse resp = e.CommandResponse;

            if (this.InvokeRequired == true)
            {
                UpdateSerialData updateHandler = new UpdateSerialData(UpdateGui);
                this.BeginInvoke(updateHandler, new Object[] { sender, e });
                return;
            }

            // Just log syntax error responses, don't process further.
            if (resp.Resp.StartsWith("Syntax error:"))
            {
                LogString(resp.CommandId + " = '" + resp.Resp + "'");
                return;
            }

            // Log all responses or only those initiated from communications log window.
            if (LogAll || this == resp.Sender)
            {
                if (laser.NeedsAuth(resp.CommandId) || loggedIn)
                    LogString("Response = '" + resp.Resp + "'");
                else
                    LogString(resp.CommandId + " = '" + resp.Resp + "'");
            }
        }

        private void listBoxCommsLog_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.C)
            {
                Clipboard.SetText(listBoxCommsLog.SelectedItem.ToString());
            }
        }
	}
}