﻿using System;
using System.Collections.Generic;
using System.Text;

using CoboltMonitor.Laser;
using System.Globalization;
using System.IO;
using System.Threading;

namespace CoboltMonitor.Gui.Logic
{
    class IPLog2Logic
    {
        class IPLogData
        {
            float _setpoint;
            float _current;
            float _power;

            public float SetPoint
            {
                get { return _setpoint; }
                set { _setpoint = value; }
            }

            public float Current
            {
                get { return _current; }
                set { _current = value; }
            }

            public float Power
            {
                get { return _power; }
                set { _power = value; }
            }

        }

        List<IPLogData> _data;
        LaserCommon laser;
        float _delay = 0.0f;
        float _logInterval = 0.0f;
        int _samples = 0;
        float _samplesInterval = 0;
        int _startdelay;
        bool _started = false;
        float _curr = 0.0f;
        float _maxp = 0.0f;
        float _savedLaserCurrent = 0.0f;
        float _startCurrent = 0.0f;
        float _endCurrent = 0.0f;

        public IPLog2Logic(LaserCommon laser)
        {
            this.laser = laser;
            _data = null;
            _started = false;
        }

        public float LogInterval
        {
            set { _logInterval = value; }
        }

        public float SamplesInterval
        {
            set { _samplesInterval = value; }
            get { return _samplesInterval; }
        }

        public float StartCurrent
        {
            set { _startCurrent = value; }
        }

        public float EndCurrent
        {
            set { _endCurrent = value; }
        }

        public int Samples
        {
            get { return _samples; }
        }

        public float GetCurrent(int idx)
        {
            return _data[idx].Current;
        }

        public float GetPower(int idx)
        {
            return _data[idx].Power;
        }

        /// <summary>
		/// Starts logging I-P data.
		/// </summary>
        public void Start()
        {
            _savedLaserCurrent = laser.LaserCurrentSetPoint;
            _samples = 0;
            _delay = 0.0f;
            _maxp = 0.0f;
            _data = new List<IPLogData>();

            SetNextCurrent();
            _startdelay = 5;		// 1 second startup delay
            _started = true;

        }

        /// <summary>
		/// Stops logging I-P data.
		/// </summary>
        public void Stop()
        {
            if (_started)
            {
                laser.SetIPLogCurrent(_savedLaserCurrent);
                Thread.Sleep(20);
                laser.SetIPLogCurrent(_savedLaserCurrent);
                Thread.Sleep(20);
                laser.SetIPLogCurrent((float)-9.0);
            }

            _started = false;

        }

        public void Abort()
        {
            _data = null;
        }

        public void WriteCSV(StreamWriter str)
        {
            foreach (IPLogData d in _data)
            {
                str.WriteLine(d.Current + ";" + d.Power);
            }
        }
        
        public bool HasData()
        {
            return (_data != null && _data.Count > 0);
        }

        /// <summary>
        /// Sets the next I/P logging current.
        /// </summary>
        public void SetNextCurrent()
        {
            float istart = _startCurrent;
            float iend = _endCurrent;
            float inext = istart + (iend - istart) * ((float)_samples) / (float)(_samplesInterval - 1);

            _curr = inext;

            laser.SetIPLogCurrent(inext);
        }
        
        /// <summary>
        /// Collects the data.
        /// </summary>
        public bool CollectData()
        {
            string pwr = laser.LaserPower.ToString(new CultureInfo("en-US"));

            if (!_started || _data == null)
                return false;

            if (_startdelay > 0)
            {
                _startdelay--;
                return false;
            }

            // Check if time to save data.
            _delay += 0.2f;
            if (_delay < _logInterval)
                return false;

            _delay = 0.0f;

            // Create new data object.
            IPLogData data = new IPLogData();
            data.Current = Convert.ToSingle(_curr, new CultureInfo("en-US"));
            data.Power = Convert.ToSingle(pwr, new CultureInfo("en-US"));

            // Save max power value.
            if (data.Power > _maxp)
                _maxp = data.Power;

            // Save data object in collection.
            _data.Add(data);

            // Update samples indicator.
            _samples++;

            return true;
        }
    }
}
