using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Configuration;
using System.Management;

namespace CoboltMonitor.Gui
{
	public partial class OptionsDlg : Form
	{
        private String serialPortName;
        private int serialBaudRate;

        public String SerialPortName
        {
            get
            {
                return serialPortName;
            }
            set
            {
                serialPortName = value;
            }
        }

        public int SerialBaudRate
        {
            get
            {
                return serialBaudRate;
            }
            set
            {
                serialBaudRate = value;
            }
        }

        public OptionsDlg()
		{
			InitializeComponent();
		}

        /// <summary>
        /// Gets the serial port names.
        /// </summary>
        /// <returns></returns>
        private List<string> GetSerialPortNames()
        {
			List<string> ports = new List<string>();
			List<string> lnames = new List<string>();
			lnames.AddRange(SerialPort.GetPortNames());

			ManagementClass c = new ManagementClass("WIN32_PNPEntity");

            foreach (ManagementObject o in c.GetInstances())
            {
                string name = (string)o["Name"];
				string deviceId = (string)o["DeviceID"];
				string manufacturer = (string)o["Manufacturer"];

				foreach (string comPortName in lnames)
				{
					if (name.Contains("(" + comPortName + ")"))
					{
						string s = comPortName + ": " + name;
						ports.Add(s);
					}
				}
            }

			return ports;
        }

        private void OptionsDlg_Activated(object sender, EventArgs e)
		{
			comboBoxPortName.Items.Clear();
			comboBoxPortName.Items.AddRange(SerialPort.GetPortNames());
			comboBoxPortName.SelectedIndex = comboBoxPortName.FindString(SerialPortName);
            
			comboBoxPortSpeed.Items.Clear();
			comboBoxPortSpeed.Items.AddRange(new object[] { 115200, 19200 });
			comboBoxPortSpeed.SelectedItem = SerialBaudRate;
		}

		private void buttonOk_Click(object sender, EventArgs e)
		{
            SerialPortName = ((string)comboBoxPortName.SelectedItem).Split(new char[] { ':' })[0];
            SerialBaudRate = (int)comboBoxPortSpeed.SelectedItem;
			Close();
		}

		private void buttonCancel_Click(object sender, EventArgs e)
		{
			Close();
		}
	}
}