﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.Globalization;
using System.IO;

using CoboltMonitor.Laser;
using CoboltMonitor.Gui.Logic;

namespace CoboltMonitor.Gui
{
	public partial class ParLog : Form
	{
        protected List<ParLogData> _data;
        float _logInterval = 0.0f;
        float _delay = 0.0f;
        bool _started = false;
        protected LaserCommon laser;

        public bool Started
        {
            get { return _started; }
            set { _started = value; }
        }

        protected class ParLogData
        {
            float _ldcurr;
            float _ldvolt;
            float _ldpow;
            float _rms;
            Tec[] _tecs;

            public float LdCurr
            {
                get { return _ldcurr; }
                set { _ldcurr = value; }
            }

            public float LdVolt
            {
                get { return _ldvolt; }
                set { _ldvolt = value; }
            }

            public float LdPow
            {
                get { return _ldpow; }
                set { _ldpow = value; }
            }

            public float Rms
            {
                get { return _rms; }
                set { _rms = value; }
            }

            public Tec[] Tecs
            {
                get { return _tecs; }
                set { _tecs = value; }
            }
            
            public ParLogData(int tecs)
            {
                _tecs = new Tec[tecs];
            }

            public override string ToString()
            {
                return LdCurr + ";" + LdVolt + ";" + LdPow + ";" + Rms;
            }
        }

        public float LogInterval
        {
            set { _logInterval = value; }
        }

        protected virtual void WriteCSV(StreamWriter str)
        {
            foreach (ParLogData d in _data)
            {
                
                str.Write(d.ToString());
                foreach (Tec t in d.Tecs)
                {
                    str.Write(";" + t.Temp + ";" + t.Drive);
                }
                str.Write(Environment.NewLine);
            }
        }
        			
        protected void CollectData(ParLogData data)
        {
            // Any IPDataCollection present to take this sample?
            if (!_started || _data == null)
                return;

            // Check if time to save data.
            _delay += 0.2f;
            if (_delay < _logInterval)
                return;

            _delay = 0.0f;

            // Save data object in collection.
            _data.Add(data);
        }

        protected virtual void NewLogData(object sender, LaserUpdatedEvent e)
        {
            Tec[] tecs = laser.GetTecs();
            switch (e.CommandResponse.CommandId)
            {
                case CommandId.ReadAll:
                    ParLogData logData = new ParLogData(tecs.Length);
                    for (int i = 0; i< logData.Tecs.Length; ++i)
                    {
                        logData.Tecs[i] = tecs[i];
                    }

                    // Laser Current, Power, Diode Volt, RMS %readings.
                    logData.LdCurr = laser.LaserCurrent;
                    logData.LdPow = laser.LaserPower;
                    logData.LdVolt = laser.LaserDiodeVoltage;
                    logData.Rms = laser.RmsPercent;

                    CollectData(logData);

                    break;
            }
        }

        public ParLog(LaserCommon laser)
		{
            _data = null;
            _started = false;

            this.laser = laser;
            laser.LaserUpdatedEvent += new EventHandler<LaserUpdatedEvent>(NewLogData);
			InitializeComponent();
		}


		private void Start()
		{
            _delay = 0.0f;
            _data = new List<ParLogData>();
            _started = true;
            numericUpDownDelay.Enabled = false;
			buttonStart.Enabled = false;
			buttonStop.Enabled = true;
		}

		private void Stop()
		{
            if (_data == null)
                return;

            if (_data.Count == 0)
                return;

            Started = false;

			numericUpDownDelay.Enabled = true;

			buttonStop.Enabled = false;
			buttonStart.Enabled = true;
		}

		private void buttonStart_Click(object sender, EventArgs e)
		{
			Start();
		}

		private void buttonStop_Click(object sender, EventArgs e)
		{
			Stop();

            Stream myStream = null;

			saveFileDialog1.Filter = "CSV (Comma delimited)|*.csv|All files (*.*)|*.*";
			saveFileDialog1.FilterIndex = 0;

			if (saveFileDialog1.ShowDialog() == DialogResult.OK)
			{
				try
				{
					if ((myStream = saveFileDialog1.OpenFile()) != null)
					{
						using (myStream)
						{
							StreamWriter writer = new StreamWriter(myStream);
                            WriteCSV(writer);
							writer.Close();
						}
					}
				}
				catch (Exception ex)
				{
                    MessageBox.Show("Failed to save file",ex.Message);
				}
			}
		}

		private void ParLog_Load(object sender, EventArgs e)
		{
			Stop();
		}

		private void ParLog_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (e.CloseReason == CloseReason.UserClosing)
				e.Cancel = true;

			Hide();
		}

        private void numericUpDownDelay_ValueChanged(object sender, EventArgs e)
        {
            LogInterval = (float)numericUpDownDelay.Value;
        }
	}
}
