﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.Globalization;
using System.IO.Ports;

namespace CoboltMonitor.Laser
{
    /// <summary>
    /// Class defining a LoggedIn on event LoggedInEvent
    /// </summary>
    public class LoggedIn : EventArgs
    {
        protected bool loggedIn;
        public LoggedIn(bool loggedin)
        {
            loggedIn = loggedin;
        }

        public bool Logged
        {
            get { return loggedIn; }
            set { loggedIn = value; }
        }
    }

    /// <summary>
    /// CoboltMonitor handles application  parts such as connecting to a laser and finding out what type of laser
    /// and also if the user is logged in or not.
    /// </summary>
    public class CoboltMonitor
    {        
        protected bool _loggedIn = false;
        public event EventHandler<LoggedIn> LoggedInEvent;
        protected string getFirmwareVersion = "gfv?";
        public CoboltMonitor()
        {

        }

        /// <summary>
        /// Sets and gets whether application is logged in or not.
        /// </summary>
        public bool LoggedIn
        {
            get { return _loggedIn; }
            set
            {
                _loggedIn = value;
                if (LoggedInEvent != null)
                    LoggedInEvent(null, new LoggedIn(_loggedIn));
            }
        }

        /// <summary>
        /// Finds connected laser and creates correct laser class.
        /// </summary>
        /// <param name="portName">The com port</param>
        /// <param name="baudRate">Selected baudrate</param>
        /// <returns>Laser object of connected laser type</returns>
        public LaserCommon ConnectLaser(string portName, int baudRate)
        {
            SerialHandler sh = new SerialHandler();
            bool retVal = sh.OpenSerialPort(portName, baudRate);

            if (retVal)
            {
                //Get the firmware version to find out connected laser type
                string data = sh.SendCommandAndWaitForResponse(getFirmwareVersion);
                if (data == String.Empty)
                {
                    sh.CloseSerialPort();
                    return null;
                }
                // if firware starts with "5." it's a Generation5 laser
                else if (data.StartsWith("5."))
                {
                    return new Gen5Laser(sh);
                }
                // else it's a Generation4 Laser
                {
                    return new Gen4Laser(sh);
                }
            }
            return null;
        }

        public LaserCommon ConnectLaserMLD(string portName, int baudRate)
        {
            SerialHandler sh = new SerialHandler();
            bool retVal = sh.OpenSerialPort(portName, baudRate);

            if (retVal)
            {
                //Get the firmware version to find out connected laser type
                string data = sh.SendCommandAndWaitForResponse(getFirmwareVersion);
                if (data == String.Empty)
                {
                    sh.CloseSerialPort();
                    return null;
                }
                // if firware starts with "6." it's a Generation6 laser
                if (data.StartsWith("6."))
                {
                    return new Gen6Laser(sh);
                }
            }
            return null;
        }
        
        public LaserCommon ConnectLaser0681(string portName, int baudRate)
        {
            SerialHandler sh = new SerialHandler();
            bool retVal = sh.OpenSerialPort(portName, baudRate);

            if (retVal)
            {
                //Get the firmware version to find out connected laser type
                string data = sh.SendCommandAndWaitForResponse(getFirmwareVersion);
                if (data == String.Empty)
                {
                    sh.CloseSerialPort();
                    return null;
                }
                // if firware starts with "6." it's a Generation6 laser
                if (data.StartsWith("6."))
                {
                    return new Gen06_81Laser(sh);
                }
            }
            return null;
        }
    }
}
